@@ -22,6 +22,9 @@
 #include "CbcStrategy.hpp"
 #ifdef COIN_HAS_CPX
 #include "OsiCpxSolverInterface.hpp"
+
+#define CHECK_CPX_STAT(a,b) if(b) throw CoinError("Error in CPLEX call",__FILE__,a);
+
 #endif
 #include "OsiAuxInfo.hpp"
 
@@ -119,8 +122,6 @@ namespace Bonmin
       addOnlyViolated_(false),
       cbcCutoffIncrement_(1e-06),
       cbcIntegerTolerance_(1e-05),
-      localSearchNodeLimit_(0),
-      maxLocalSearchPerNode_(0),
       maxLocalSearch_(0),
       maxLocalSearchTime_(3600),
       subMilpLogLevel_(0),
@@ -141,8 +142,6 @@ namespace Bonmin
       addOnlyViolated_(other.addOnlyViolated_),
       cbcCutoffIncrement_(other.cbcCutoffIncrement_),
       cbcIntegerTolerance_(other.cbcIntegerTolerance_),
-      localSearchNodeLimit_(other.localSearchNodeLimit_),
-      maxLocalSearchPerNode_(other.maxLocalSearchPerNode_),
       maxLocalSearch_(other.maxLocalSearch_),
       maxLocalSearchTime_(other.maxLocalSearchTime_),
       subMilpLogLevel_(other.subMilpLogLevel_),
@@ -203,8 +202,7 @@ namespace Bonmin
 
 
   void
-  OaDecompositionBase::SubMipSolver::performLocalSearch(double cutoff, int loglevel, double maxTime,
-      int maxNodes)
+  OaDecompositionBase::SubMipSolver::performLocalSearch(double cutoff, int loglevel, double maxTime)
   {
     if (clp_) {
       if (!strategy_)
@@ -224,7 +222,6 @@ namespace Bonmin
       cbc_->setStrategy(*strategy_);
       cbc_->setLogLevel(loglevel);
       cbc_->solver()->messageHandler()->setLogLevel(0);
-      cbc_->setMaximumNodes(maxNodes);
       cbc_->setMaximumSeconds(maxTime);
       cbc_->setCutoff(cutoff);
       cbc_->branchAndBound();
@@ -252,7 +249,6 @@ namespace Bonmin
 #ifdef COIN_HAS_CPX
       if (cpx_) {
         CPXENVptr env = cpx_->getEnvironmentPtr();
-        CPXsetintparam(env, CPX_PARAM_NODELIM, maxNodes);
         CPXsetdblparam(env, CPX_PARAM_TILIM, maxTime);
         CPXsetdblparam(env, CPX_PARAM_CUTUP, cutoff);
         //CpxModel = cpx_;
@@ -265,26 +261,52 @@ namespace Bonmin
             "OaDecompositionBase::SubMipSolver");
     }
 
-    lp_->branchAndBound();
-
-   optimal_ = lp_->isProvenOptimal();
 #ifdef COIN_HAS_CPX
     if (cpx_) {
+      cpx_->switchToMIP();
       //CpxModel = NULL;
       CPXENVptr env = cpx_->getEnvironmentPtr();
       CPXLPptr cpxlp = cpx_->getLpPtr(OsiCpxSolverInterface::KEEPCACHED_ALL);
 
-       
-      int status = CPXgetbestobjval(env, cpxlp, &lowBound_);
-     
+      int status = CPXmipopt(env,cpxlp);
+      CHECK_CPX_STAT("mipopt",status)
+
       int stat = CPXgetstat( env, cpxlp);
-      optimal_ |= (stat == CPXMIP_INFEASIBLE); 
-       nodeCount_ = CPXgetnodecnt(env , cpxlp);
-      iterationCount_ = CPXgetmipitcnt(env , cpxlp);
-      if (status)
-        throw CoinError("Error in getting some CPLEX information","OaDecompositionBase::SubMipSolver","performLocalSearch");
-    }
+//#define FOR_HASSAN
+#ifdef FOR_HASSAN
+      std::cout << "STATUS = "<<stat<<"\n";
+      if(stat==119){
+          cpx_->writeMpsNative("OA.mps", NULL, NULL, 1);
+          throw CoinError("Ok program stoped by me !","OaDecompositionBase::SubMipSolver","performLocalSearch");
+      }
 #endif
+      bool infeasible = (stat == CPXMIP_INFEASIBLE) || (stat == CPXMIP_ABORT_INFEAS) || (stat == CPXMIP_TIME_LIM_INFEAS) || (stat == CPXMIP_NODE_LIM_INFEAS) || (stat == CPXMIP_FAIL_INFEAS)
+                        || (stat == CPXMIP_MEM_LIM_INFEAS) || (stat == CPXMIP_INForUNBD);
+      optimal_ = (stat == CPXMIP_INFEASIBLE) || (stat == CPXMIP_OPTIMAL) || (stat == CPXMIP_OPTIMAL_TOL); 
+      nodeCount_ = CPXgetnodecnt(env , cpxlp);
+      iterationCount_ = CPXgetmipitcnt(env , cpxlp);
+      status = CPXgetbestobjval(env, cpxlp, &lowBound_);
+      CHECK_CPX_STAT("getbestobjval",status)
+       
+      if(!infeasible){
+         if(!integerSolution_){
+           integerSolution_ = new double[lp_->getNumCols()];
+         }
+         CPXgetmipx(env, cpxlp, integerSolution_, 0, lp_->getNumCols() -1);
+         CHECK_CPX_STAT("getmipx",status)
+      }
+      else {
+        if (integerSolution_) {
+          delete [] integerSolution_;
+          integerSolution_ = NULL;
+        }
+      }
+    }
+    else {
+#endif
+    lp_->branchAndBound();
+
+   optimal_ = lp_->isProvenOptimal();
 
     if (lp_->getFractionalIndices().size() == 0) {
       if (!integerSolution_)
@@ -296,6 +318,9 @@ namespace Bonmin
       integerSolution_ = NULL;
     }
   }
+#ifdef COIN_HAS_CPX
+  }
+#endif
 }
 
 OaDecompositionBase::solverManip::solverManip
