@@ -286,6 +286,7 @@ namespace Bonmin
       for (int i=0; i<nco; i++) 
 	objs [i] = s.continuousSolver () -> objects () [i];
       model_.addObjects (nco, objs);
+      delete [] objs;
     }
 
     CbcBranchDefaultDecision branch;
@@ -377,6 +378,8 @@ namespace Bonmin
 
     model_.setMaximumNodes(s.getIntParameter(BabSetupBase::MaxNodes));
 
+    model_.setMaximumNumberIterations(s.getIntParameter(BabSetupBase::MaxIterations));
+
     model_.setMaximumSolutions(s.getIntParameter(BabSetupBase::MaxSolutions));
 
     model_.setIntegerTolerance(s.getDoubleParameter(BabSetupBase::IntTol));
@@ -425,8 +428,19 @@ namespace Bonmin
       }
     }
 
+   try {
     //Get the time and start.
     model_.initialSolve();
+
+    //Added by Claudia
+    OsiTMINLPInterface * nlpSolver = dynamic_cast<OsiTMINLPInterface *>(model_.solver());
+    if(nlpSolver && nlpSolver->getNewCutoffDecr()!=COIN_DBL_MAX)
+      model_.setDblParam(CbcModel::CbcCutoffIncrement, nlpSolver->getNewCutoffDecr());
+
+    // for Couenne
+    if (usingCouenne_)
+      model_.passInSolverCharacteristics (bonBabInfoPtr);
+
     continuousRelaxation_ =model_.solver()->getObjValue();
     if (specOpt==16)//Set warm start point for Ipopt
     {
@@ -454,7 +468,15 @@ namespace Bonmin
     // to get node parent info in Cbc, pass parameter 3.
     //model_.branchAndBound(3);
     model_.branchAndBound();
-    
+    }
+    catch(TNLPSolver::UnsolvedError *E){
+      s.nonlinearSolver()->model()->finalize_solution(TMINLP::MINLP_ERROR,
+           0,
+           NULL,
+           DBL_MAX);
+      throw E;
+   
+    }
     numNodes_ = model_.getNodeCount();
     bestObj_ = model_.getObjValue();
     bestBound_ = model_.getBestPossibleObjValue();
@@ -515,10 +537,10 @@ namespace Bonmin
     if (model_.numberObjects()==0) {
       if (bestSolution_)
         delete [] bestSolution_;
-      bestSolution_ = new double[s.nonlinearSolver()->getNumCols()];
-      CoinCopyN(s.nonlinearSolver()->getColSolution(), s.nonlinearSolver()->getNumCols(),
-          bestSolution_);
-      bestObj_ = bestBound_ = s.nonlinearSolver()->getObjValue();
+      OsiSolverInterface * solver = 
+             (s.nonlinearSolver() == s.continuousSolver())? 
+             model_.solver() : s.nonlinearSolver();
+      bestObj_ = bestBound_ = solver->getObjValue();
     }
 
     if (bonBabInfoPtr->bestSolution2().size() > 0) {
@@ -539,6 +561,11 @@ namespace Bonmin
       CoinCopyN(model_.bestSolution(), s.nonlinearSolver()->getNumCols(), bestSolution_);
     }
     if (model_.status() == 0) {
+      if(model_.isContinuousUnbounded()){
+        status = TMINLP::CONTINUOUS_UNBOUNDED;
+        mipStatus_ = UnboundedOrInfeasible;
+      }
+      else
       if (bestSolution_) {
         status = TMINLP::SUCCESS;
         mipStatus_ = FeasibleOptimal;
