@@ -1,8 +1,8 @@
-// Copyright (C) 2005, 2007 International Business Machines and others.
+// Copyright (C) 2005, 2009 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpLowRankAugSystemSolver.cpp 1327 2008-09-18 19:01:17Z andreasw $
+// $Id: IpLowRankAugSystemSolver.cpp 1578 2009-10-06 23:40:31Z andreasw $
 //
 // Authors:  Andreas Waechter                IBM    2005-12-27
 
@@ -93,6 +93,13 @@ namespace Ipopt
       Wdiag_ = Wdiag_space->MakeNewDiagMatrix();
     }
 
+    // This might be used with a linear solver that cannot detect the
+    // inertia.  In that case, we should not asked for checking the
+    // number of negative eigenvalues.
+    if (!aug_system_solver_->ProvidesInertia()) {
+      check_NegEVals = false;
+    }
+
     if (first_call_ ||
         AugmentedSystemRequiresChange(W, W_factor, D_x, delta_x, D_s, delta_s,
                                       *J_c, D_c, delta_c, *J_d, D_d,
