@@ -1,13 +1,18 @@
-// Copyright (C) 2004, 2006 International Business Machines and others.
+// Copyright (C) 2004, 2009 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: AmplTNLP.cpp 759 2006-07-07 03:07:08Z andreasw $
+// $Id: AmplTNLP.cpp 1578 2009-10-06 23:40:31Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
 #include "IpoptConfig.h"
-#include "config_ipopt.h"
+#ifdef HAVE_CONFIG_H
+# include "config_ipopt.h"
+#else
+# define PACKAGE_VERSION IPOPT_VERSION
+#endif
+
 #include "AmplTNLP.hpp"
 #include "IpDenseVector.hpp"
 #include "IpGenTMatrix.hpp"
@@ -21,7 +26,7 @@
 
 namespace Ipopt
 {
-#ifdef IP_DEBUG
+#if COIN_IPOPT_VERBOSITY > 0
   static const Index dbg_verbosity = 0;
 #endif
 
@@ -41,7 +46,6 @@ namespace Ipopt
       asl_(NULL),
       obj_sign_(1),
       nz_h_full_(-1),
-      non_const_x_(NULL),
       x_sol_(NULL),
       z_L_sol_(NULL),
       z_U_sol_(NULL),
@@ -50,6 +54,8 @@ namespace Ipopt
       obj_sol_(0.0),
       objval_called_with_current_x_(false),
       conval_called_with_current_x_(false),
+      hesset_called_(false),
+      set_active_objective_called_(false),
       Oinfo_ptr_(NULL),
       suffix_handler_(suffix_handler)
   {
@@ -79,12 +85,12 @@ namespace Ipopt
     FILE*nl = NULL;
     if (nl_file_content) {
       nl = jac0dim(const_cast<char*>(nl_file_content->c_str()),
-                   -nl_file_content->length());
+                   -(ftnlen)nl_file_content->length());
     }
     else {
       if (!stub) {
         jnlst_->Printf(J_ERROR, J_MAIN, "No .nl file given!\n");
-        exit(-1);
+        THROW_EXCEPTION(INVALID_TNLP, "No .nl file given!\n");
       }
       nl = jac0dim(stub, (fint)strlen(stub));
       DBG_ASSERT(nl);
@@ -103,8 +109,7 @@ namespace Ipopt
                      "Discrete variables not allowed when the allow_discrete flag is false, "
                      "Either remove the integer variables, or change the flag in the constructor of AmplTNLP"
                     );
-    // n_con can be >= 0
-    DBG_ASSERT(n_obj <= 1); // Currently can handle only 0 or 1 objective
+
     DBG_ASSERT(nlo == 0 || nlo == 1); // Can handle nonlinear obj.
     DBG_ASSERT(nwv == 0); // Don't know what "linear arc" variables are
     DBG_ASSERT(nlnc == 0); // Don't know what "nonlinear network"constraints are
@@ -112,8 +117,8 @@ namespace Ipopt
 
     // Set options in the asl structure
     want_xpi0 = 1 | 2;  // allocate initial values for primal and dual if available
+    obj_no = 0;
     DBG_ASSERT((want_xpi0 & 1) == 1 && (want_xpi0 & 2) == 2);
-    obj_no = 0; // always want to work with the first (and only?) objective
 
     // allocate space for initial values
     X0 = new real[n_var];
@@ -130,65 +135,98 @@ namespace Ipopt
     int retcode = pfgh_read(nl, ASL_return_read_err | ASL_findgroups);
 
     switch (retcode) {
-      case ASL_readerr_none : {}
+    case ASL_readerr_none : {}
       break;
-      case ASL_readerr_nofile : {
+    case ASL_readerr_nofile : {
         jnlst_->Printf(J_ERROR, J_MAIN, "Cannot open .nl file\n");
-        exit(-1);
+        THROW_EXCEPTION(INVALID_TNLP, "Cannot open .nl file");
       }
       break;
-      case ASL_readerr_nonlin : {
+    case ASL_readerr_nonlin : {
         DBG_ASSERT(false); // this better not be an error!
         jnlst_->Printf(J_ERROR, J_MAIN, "model involves nonlinearities (ed0read)\n");
-        exit(-1);
+        THROW_EXCEPTION(INVALID_TNLP, "model involves nonlinearities (ed0read)");
       }
       break;
-      case  ASL_readerr_argerr : {
+    case  ASL_readerr_argerr : {
         jnlst_->Printf(J_ERROR, J_MAIN, "user-defined function with bad args\n");
-        exit(-1);
+        THROW_EXCEPTION(INVALID_TNLP, "user-defined function with bad args");
       }
       break;
-      case ASL_readerr_unavail : {
+    case ASL_readerr_unavail : {
         jnlst_->Printf(J_ERROR, J_MAIN, "user-defined function not available\n");
-        exit(-1);
+        THROW_EXCEPTION(INVALID_TNLP, "user-defined function not available");
       }
       break;
-      case ASL_readerr_corrupt : {
+    case ASL_readerr_corrupt : {
         jnlst_->Printf(J_ERROR, J_MAIN, "corrupt .nl file\n");
-        exit(-1);
+        THROW_EXCEPTION(INVALID_TNLP, "corrupt .nl file");
       }
       break;
-      case ASL_readerr_bug : {
+    case ASL_readerr_bug : {
         jnlst_->Printf(J_ERROR, J_MAIN, "bug in .nl reader\n");
-        exit(-1);
+        THROW_EXCEPTION(INVALID_TNLP, "bug in .nl reader");
       }
       break;
-      case ASL_readerr_CLP : {
-        jnlst_->Printf(J_ERROR, J_MAIN, "ASL error message: \"solver cannot handle CLP extensions\"\n");
-        exit(-1);
+    case ASL_readerr_CLP : {
+        jnlst_->Printf(J_ERROR, J_MAIN, "Ampl model contains a constraint without \"=\", \">=\", or \"<=\".\n");
+        THROW_EXCEPTION(INVALID_TNLP, "Ampl model contains a constraint without \"=\", \">=\", or \"<=\".");
       }
       break;
-      default: {
+    default: {
         jnlst_->Printf(J_ERROR, J_MAIN, "Unknown error in stub file read. retcode = %d\n", retcode);
-        exit(-1);
+        THROW_EXCEPTION(INVALID_TNLP, "Unknown error in stub file read");
       }
       break;
     }
+  }
 
-    // see "changes" in solvers directory of ampl code...
-    hesset(1,0,1,0,nlc);
+  void AmplTNLP::set_active_objective(Index in_obj_no)
+  {
+    if (hesset_called_) {
+      jnlst_->Printf(J_ERROR, J_MAIN, "Internal error: AmplTNLP::set_active_objective called after AmplTNLP::call_hesset.\n");
+      THROW_EXCEPTION(INVALID_TNLP, "Internal error: AmplTNLP::set_active_objective called after AmplTNLP::call_hesset.");
+    }
+    ASL_pfgh* asl = asl_;
+    obj_no = in_obj_no;
+    set_active_objective_called_ = true;
+  }
+
+  void AmplTNLP::call_hesset()
+  {
+    if (hesset_called_) {
+      jnlst_->Printf(J_ERROR, J_MAIN, "Internal error: AmplTNLP::call_hesset is called twice.\n");
+      THROW_EXCEPTION(INVALID_TNLP, "Internal error: AmplTNLP::call_hesset is called twice.");
+    }
+
+    ASL_pfgh* asl = asl_;
+
+    if (n_obj == 0) {
+      hesset(1,0,0,0,nlc);
+    }
+    else {
+      if (n_obj>1 && !set_active_objective_called_) {
+        jnlst_->Printf(J_ERROR, J_MAIN,
+                       "There is more than one objective function in the AMPL model, but AmplTNLP::set_active_objective has not been called.\n");
+        THROW_EXCEPTION(INVALID_TNLP, "There is more than one objective function in the AMPL model, but AmplTNLP::set_active_objective has not been called");
+      }
+      // see "changes" in solvers directory of ampl code...
+      hesset(1,obj_no,1,0,nlc);
+    }
 
     obj_sign_ = 1; // minimization
     if (objtype[obj_no] != 0) {
       obj_sign_ = -1;
     }
 
-    // find the nonzero structure for the hessian
-    // parameters to sphsetup:
-    int coeff_obj = 1; // coefficient of the objective fn ???
+    // find the nonzero structure for the hessian parameters to
+    // sphsetup:
+    int coeff_obj = 1;
     int mult_supplied = 1; // multipliers will be supplied
     int uptri = 1; // only need the upper triangular part
     nz_h_full_ = sphsetup(-1, coeff_obj, mult_supplied, uptri);
+
+    hesset_called_ = true;
   }
 
   AmplTNLP::~AmplTNLP()
@@ -217,8 +255,6 @@ namespace Ipopt
       asl_ = NULL;
     }
 
-    delete [] non_const_x_;
-    non_const_x_ = NULL;
     delete [] x_sol_;
     x_sol_ = NULL;
     delete [] z_L_sol_;
@@ -247,6 +283,10 @@ namespace Ipopt
     ASL_pfgh* asl = asl_;
     DBG_ASSERT(asl_);
 
+    if (!hesset_called_) {
+      call_hesset();
+    }
+
     n = n_var; // # of variables (variable types have been asserted in the constructor
     m = n_con; // # of constraints
     nnz_jac_g = nzc; // # of non-zeros in the jacobian
@@ -257,6 +297,52 @@ namespace Ipopt
     return true;
   }
 
+  bool AmplTNLP::get_var_con_metadata(Index n,
+                                      StringMetaDataMapType& var_string_md,
+                                      IntegerMetaDataMapType& var_integer_md,
+                                      NumericMetaDataMapType& var_numeric_md,
+                                      Index m,
+                                      StringMetaDataMapType& con_string_md,
+                                      IntegerMetaDataMapType& con_integer_md,
+                                      NumericMetaDataMapType& con_numeric_md)
+  {
+    ASL_pfgh* asl = asl_;
+    DBG_ASSERT(asl_);
+
+    // pick up the variable and constraints names if available
+    Index rlen = maxrownamelen;
+    Index clen = maxcolnamelen;
+
+    if (rlen > 0) {
+      std::vector<std::string> var_names(n);
+      for (Index i=0; i<n; i++) {
+        var_names[i] = var_name(i);
+      }
+      var_string_md_["idx_names"] = var_names;
+    }
+
+    if (clen > 0) {
+      std::vector<std::string> con_names(m);
+      for (Index i=0; i<m; i++) {
+        con_names[i] = con_name(i);
+      }
+      con_string_md_["idx_names"] = con_names;
+    }
+
+    if (var_string_md_.size() > 0 || var_integer_md_.size() > 0 || var_numeric_md_.size() > 0
+        || con_string_md_.size() > 0 || con_integer_md_.size() > 0 || con_numeric_md_.size() > 0) {
+      var_string_md = var_string_md_;
+      var_integer_md = var_integer_md_;
+      var_numeric_md = var_numeric_md_;
+      con_string_md = con_string_md_;
+      con_integer_md = con_integer_md_;
+      con_numeric_md = con_numeric_md_;
+      return true;
+    }
+
+    return false;
+  }
+
   bool AmplTNLP::get_bounds_info(Index n, Number* x_l, Number* x_u, Index m, Number* g_l, Number* g_u)
   {
     ASL_pfgh* asl = asl_;
@@ -278,6 +364,25 @@ namespace Ipopt
     return true;
   }
 
+
+  bool AmplTNLP::get_constraints_linearity(Index n,
+      LinearityType* const_types)
+  {
+    ASL_pfgh* asl = AmplSolverObject();
+    //check that n is good
+    DBG_ASSERT(n == n_con);
+    // check that there are no network constraints
+    DBG_ASSERT(nlnc == 0 && lnc == 0);
+    //the first nlc constraints are non linear the rest is linear
+    for (Index i=0; i<nlc; i++) {
+      const_types[i]=NON_LINEAR;
+    }
+    // the rest is linear
+    for (Index i=nlc; i<n_con; i++)
+      const_types[i]=LINEAR;
+    return true;
+  }
+
   bool AmplTNLP::get_starting_point(Index n, bool init_x, Number* x, bool init_z, Number* z_L, Number* z_U, Index m, bool init_lambda, Number* lambda)
   {
     ASL_pfgh* asl = asl_;
@@ -297,8 +402,23 @@ namespace Ipopt
     }
 
     if (init_z) {
+      // Modified for warm-start from AMPL
+      DBG_ASSERT(IsValid(suffix_handler_));
+      const double* zL_init = suffix_handler_->GetNumberSuffixValues("ipopt_zL_in", AmplSuffixHandler::Variable_Source);
+      const double* zU_init = suffix_handler_->GetNumberSuffixValues("ipopt_zU_in", AmplSuffixHandler::Variable_Source);
       for (Index i=0; i<n; i++) {
-        z_L[i] = z_U[i] = 1.0;
+        if (zL_init) {
+          z_L[i]=zL_init[i];
+        }
+        else {
+          z_L[i] =1.0;
+        }
+        if (zU_init) {
+          z_U[i]=zU_init[i];
+        }
+        else {
+          z_U[i] =1.0;
+        }
       }
     }
 
@@ -324,7 +444,7 @@ namespace Ipopt
       return false;
     }
 
-    return internal_objval(obj_value);
+    return internal_objval(x, obj_value);
   }
 
   bool AmplTNLP::eval_grad_f(Index n, const Number* x, bool new_x, Number* grad_f)
@@ -344,7 +464,7 @@ namespace Ipopt
       }
     }
     else {
-      objgrd(0, non_const_x_, grad_f, (fint*)nerror_);
+      objgrd(obj_no, const_cast<Number*>(x), grad_f, (fint*)nerror_);
       if (!nerror_ok(nerror_)) {
         return false;
       }
@@ -361,18 +481,16 @@ namespace Ipopt
   bool AmplTNLP::eval_g(Index n, const Number* x, bool new_x, Index m, Number* g)
   {
     DBG_START_METH("AmplTNLP::eval_g", dbg_verbosity);
-#ifdef IP_DEBUG
 
-    ASL_pfgh* asl = asl_;
+    DBG_DO(ASL_pfgh* asl = asl_);
     DBG_ASSERT(n == n_var);
     DBG_ASSERT(m == n_con);
-#endif
 
     if (!apply_new_x(new_x, n, x)) {
       return false;
     }
 
-    return internal_conval(m, g);
+    return internal_conval(x, m, g);
   }
 
   bool AmplTNLP::eval_jac_g(Index n, const Number* x, bool new_x,
@@ -393,8 +511,8 @@ namespace Ipopt
         for (cgrad* cg=Cgrad[i]; cg; cg = cg->next) {
           iRow[cg->goff] = i + 1;
           jCol[cg->goff] = cg->varno + 1;
-          //				iRow[current_nz] = i + 1;
-          //				jCol[current_nz] = cg->varno+1;
+          //    iRow[current_nz] = i + 1;
+          //    jCol[current_nz] = cg->varno+1;
           current_nz++;
         }
       }
@@ -406,7 +524,7 @@ namespace Ipopt
         return false;
       }
 
-      jacval(non_const_x_, values, (fint*)nerror_);
+      jacval(const_cast<Number*>(x), values, (fint*)nerror_);
       if (nerror_ok(nerror_)) {
         return true;
       }
@@ -449,23 +567,22 @@ namespace Ipopt
       }
       if (!objval_called_with_current_x_) {
         Number dummy;
-        internal_objval(dummy);
-        internal_conval(m);
+        internal_objval(x, dummy);
+        internal_conval(x, m);
       }
       if (!conval_called_with_current_x_) {
-        internal_conval(m);
-      }
-      // copy lambda to non_const_lambda - note, we do not store a copy like
-      // we do with x since lambda is only used here and not in other calls
-      Number* non_const_lambda = new Number[m];
-      for (Index i=0; i<m; i++) {
-        non_const_lambda[i] = lambda[i];
+        internal_conval(x, m);
       }
 
-      real ow=obj_sign_*obj_factor;
-      sphes(values, -1, &ow, non_const_lambda);
-
-      delete [] non_const_lambda;
+      real* OW = new real[Max(1,n_obj)];
+      if (n_obj>0) {
+        for (Index i=0; i<n_obj; i++) {
+          OW[i] = 0.;
+        }
+        OW[obj_no] = obj_sign_*obj_factor;
+      }
+      sphes(values, -1, OW, const_cast<Number*>(lambda));
+      delete [] OW;
       return true;
     }
     else {
@@ -478,7 +595,9 @@ namespace Ipopt
   void AmplTNLP::finalize_solution(SolverReturn status,
                                    Index n, const Number* x, const Number* z_L, const Number* z_U,
                                    Index m, const Number* g, const Number* lambda,
-                                   Number obj_value)
+                                   Number obj_value,
+                                   const IpoptData* ip_data,
+                                   IpoptCalculatedQuantities* ip_cq)
   {
     ASL_pfgh* asl = asl_;
 
@@ -514,6 +633,10 @@ namespace Ipopt
       message = "Maximum Number of Iterations Exceeded.";
       solve_result_num = 400;
     }
+    else if (status == CPUTIME_EXCEEDED) {
+      message = "Maximum CPU Time Exceeded.";
+      solve_result_num = 401;
+    }
     else if (status == STOP_AT_TINY_STEP) {
       message = "Search Direction becomes Too Small.";
       solve_result_num = 500;
@@ -522,6 +645,10 @@ namespace Ipopt
       message = "Solved To Acceptable Level.";
       solve_result_num = 1;
     }
+    else if (status == FEASIBLE_POINT_FOUND) {
+      message = "Found feasible point for square problem.";
+      solve_result_num = 2;
+    }
     else if (status == LOCAL_INFEASIBILITY) {
       message = "Converged to a locally infeasible point. Problem may be infeasible.";
       solve_result_num = 200;
@@ -539,12 +666,18 @@ namespace Ipopt
       solve_result_num = 502;
     }
 
+    if (IsValid(suffix_handler_)) {
+      // Modified for warm-start from AMPL. Assign Bound Multipliers as Suffixes
+      suf_rput("ipopt_zL_out", ASL_Sufkind_var,  z_L_sol_);
+      suf_rput("ipopt_zU_out", ASL_Sufkind_var,  z_U_sol_);
+    }
+
     // Write the .sol file
-    message = " \n" PACKAGE_STRING ": " + message;
+    message = " \nIpopt " PACKAGE_VERSION ": " + message;
     write_solution_file(message.c_str());
   }
 
-  bool AmplTNLP::internal_objval(Number& obj_val)
+  bool AmplTNLP::internal_objval(const Number* x, Number& obj_val)
   {
     DBG_START_METH("AmplTNLP::internal_objval",
                    dbg_verbosity);
@@ -558,7 +691,7 @@ namespace Ipopt
       return true;
     }
     else {
-      Number retval = objval(0, non_const_x_, (fint*)nerror_);
+      Number retval = objval(obj_no, const_cast<Number*>(x), (fint*)nerror_);
       if (nerror_ok(nerror_)) {
         obj_val = obj_sign_*retval;
         objval_called_with_current_x_ = true;
@@ -569,7 +702,7 @@ namespace Ipopt
     return false;
   }
 
-  bool AmplTNLP::internal_conval(Index m, Number* g)
+  bool AmplTNLP::internal_conval(const Number* x, Index m, Number* g)
   {
     DBG_START_METH("AmplTNLP::internal_conval",
                    dbg_verbosity);
@@ -583,8 +716,7 @@ namespace Ipopt
       g = new double[m];
       allocated = true;
     }
-
-    conval(non_const_x_, g, (fint*)nerror_);
+    conval(const_cast<Number*>(x), g, (fint*)nerror_);
 
     if (allocated) {
       delete [] g;
@@ -608,23 +740,18 @@ namespace Ipopt
     DBG_ASSERT(asl_);
 
     if (new_x) {
+      if (!hesset_called_) {
+        call_hesset();
+      }
+
       DBG_PRINT((1, "Set new x.\n"));
       // update the flags so these methods are called
       // before evaluating the hessian
       conval_called_with_current_x_ = false;
       objval_called_with_current_x_ = false;
 
-      //copy the data to the non_const_x_
-      if (!non_const_x_) {
-        non_const_x_ = new Number[n];
-      }
-
-      for (Index i=0; i<n; i++) {
-        non_const_x_[i] = x[i];
-      }
-
       // tell ampl that we have a new x
-      xknowne(non_const_x_, (fint*)nerror_);
+      xknowne(const_cast<Number*>(x), (fint*)nerror_);
       return nerror_ok(nerror_);
     }
 
@@ -748,7 +875,7 @@ namespace Ipopt
       if (!pinfo->Options()->SetNumericValue(pinfo->IpoptName().c_str(), real_val)) {
         pinfo->Jnlst()->Printf(J_ERROR, J_MAIN,
                                "\nInvalid value \"%s\" for option %s.\n", value, kw->name);
-        exit(-1);
+        THROW_EXCEPTION(OPTION_INVALID, "Invalid numeric option");
       }
 
       return retval;
@@ -766,7 +893,7 @@ namespace Ipopt
       if (!pinfo->Options()->SetIntegerValue(pinfo->IpoptName().c_str(), int_val)) {
         pinfo->Jnlst()->Printf(J_ERROR, J_MAIN,
                                "\nInvalid value \"%s\" for option %s.\n", value, kw->name);
-        exit(-1);
+        THROW_EXCEPTION(OPTION_INVALID, "Invalid integer option");
       }
 
       return retval;
@@ -784,7 +911,7 @@ namespace Ipopt
       if (!pinfo->Options()->SetStringValue(pinfo->IpoptName().c_str(), str_val)) {
         pinfo->Jnlst()->Printf(J_ERROR, J_MAIN,
                                "\nInvalid value \"%s\" for option %s.\n", value, kw->name);
-        exit(-1);
+        THROW_EXCEPTION(OPTION_INVALID, "Invalid string option");
       }
 
       return retval;
@@ -813,7 +940,7 @@ namespace Ipopt
       else {
         pinfo->Jnlst()->Printf(J_ERROR, J_MAIN,
                                "\nInvalid value \"%s\" for option %s.\n", value, kw->name);
-        exit(-1);
+        THROW_EXCEPTION(OPTION_INVALID, "Invalid option");
       }
 
       return retval;
@@ -861,29 +988,29 @@ namespace Ipopt
       strcpy(keywords[ioption].name, iter->first.c_str());
       keywords[ioption].desc = iter->second->Description();
       switch (iter->second->Type()) {
-        case String_Option: {
+      case String_Option: {
           PrivatInfo* pinfo = new PrivatInfo(iter->second->IpoptOptionName(), options, jnlst);
           keywords[ioption].info = (void*) pinfo;
           keywords[ioption].kf = get_str_opt;
         }
         break;
-        case Number_Option: {
+      case Number_Option: {
           PrivatInfo* pinfo = new PrivatInfo(iter->second->IpoptOptionName(), options, jnlst);
           keywords[ioption].info = (void*) pinfo;
           keywords[ioption].kf = get_num_opt;
         }
         break;
-        case Integer_Option: {
+      case Integer_Option: {
           PrivatInfo* pinfo = new PrivatInfo(iter->second->IpoptOptionName(), options, jnlst);
           keywords[ioption].info = (void*) pinfo;
           keywords[ioption].kf = get_int_opt;
         }
         break;
-        case WS_Option:
+      case WS_Option:
         keywords[ioption].info = NULL;
         keywords[ioption].kf = WS_val;
         break;
-        case HaltOnError_Option:
+      case HaltOnError_Option:
         PrivatInfo* pinfo = new PrivatInfo(iter->second->IpoptOptionName(), options, jnlst, nerror);
         keywords[ioption].info = (void*) pinfo;
         keywords[ioption].kf = get_haltonerror_opt;
@@ -936,6 +1063,10 @@ namespace Ipopt
                                      "file_print_level",
                                      AmplOptionsList::Integer_Option,
                                      "Verbosity level for output file");
+    ampl_options_list->AddAmplOption("option_file_name",
+                                     "option_file_name",
+                                     AmplOptionsList::String_Option,
+                                     "File name of options file (default: ipopt.opt)");
 
     // Termination
     ampl_options_list->AddAmplOption("tol",
@@ -950,6 +1081,10 @@ namespace Ipopt
                                      "max_iter",
                                      AmplOptionsList::Integer_Option,
                                      "Maximum number of iterations (same as max_iter)");
+    ampl_options_list->AddAmplOption("max_cpu_time",
+                                     "max_cpu_time",
+                                     AmplOptionsList::Number_Option,
+                                     "CPU time limit");
     ampl_options_list->AddAmplOption("compl_inf_tol",
                                      "compl_inf_tol",
                                      AmplOptionsList::Number_Option,
@@ -1017,6 +1152,11 @@ namespace Ipopt
                                      "mu_oracle",
                                      AmplOptionsList::String_Option,
                                      "Oracle for a new barrier parameter in the adaptive strategy");
+    // Barrier parameter
+    ampl_options_list->AddAmplOption("mu_max",
+                                     "mu_max",
+                                     AmplOptionsList::Number_Option,
+                                     "Maximal value for barrier parameter for adaptive strategy");
     ampl_options_list->AddAmplOption("mu_init",
                                      "mu_init",
                                      AmplOptionsList::Number_Option,
@@ -1031,6 +1171,14 @@ namespace Ipopt
                                      "bound_push",
                                      AmplOptionsList::Number_Option,
                                      "Desired minimal absolute distance of initial point to bound");
+    ampl_options_list->AddAmplOption("slack_bound_frac",
+                                     "slack_bound_frac",
+                                     AmplOptionsList::Number_Option,
+                                     "Desired minimal relative distance of initial slack to bound");
+    ampl_options_list->AddAmplOption("slack_bound_push",
+                                     "slack_bound_push",
+                                     AmplOptionsList::Number_Option,
+                                     "Desired minimal absolute distance of initial slack to bound");
     ampl_options_list->AddAmplOption("bound_mult_init_val",
                                      "bound_mult_init_val",
                                      AmplOptionsList::Number_Option,
@@ -1066,7 +1214,25 @@ namespace Ipopt
                                      AmplOptionsList::Number_Option,
                                      "Required infeasibility reduction in restoration phase");
 
+    // Added for Warm-Start
+    ampl_options_list->AddAmplOption("warm_start_init_point",
+                                     "warm_start_init_point",
+                                     AmplOptionsList::String_Option,
+                                     "Enables to specify bound multiplier values");
+    ampl_options_list->AddAmplOption("warm_start_bound_push",
+                                     "warm_start_bound_push",
+                                     AmplOptionsList::Number_Option,
+                                     "Enables to specify how much should variables should be pushed inside the feasible region");
+    ampl_options_list->AddAmplOption("warm_start_mult_bound_push",
+                                     "warm_start_mult_bound_push",
+                                     AmplOptionsList::Number_Option,
+                                     "Enables to specify how much should bound multipliers should be pushed inside the feasible region");
 
+    // Quasi-Newton
+    ampl_options_list->AddAmplOption("hessian_approximation",
+                                     "hessian_approximation",
+                                     AmplOptionsList::String_Option,
+                                     "Can enable Quasi-Newton approximation of hessian");
     // Linear solver
     ampl_options_list->AddAmplOption("linear_solver",
                                      "linear_solver",
@@ -1097,8 +1263,6 @@ namespace Ipopt
                                      "Can enable Quasi-Newton approximation of hessian");
 
     // Special linear solver options
-#ifdef HAVE_MA27
-
     ampl_options_list->AddAmplOption("ma27_pivtol",
                                      "ma27_pivtol",
                                      AmplOptionsList::Number_Option,
@@ -1107,9 +1271,6 @@ namespace Ipopt
                                      "ma27_pivtolmax",
                                      AmplOptionsList::Number_Option,
                                      "Maximal pivot tolerance for the linear solver MA27");
-#endif
-
-#ifdef HAVE_MA57
 
     ampl_options_list->AddAmplOption("ma57_pivtol",
                                      "ma57_pivtol",
@@ -1119,9 +1280,6 @@ namespace Ipopt
                                      "ma57_pivtolmax",
                                      AmplOptionsList::Number_Option,
                                      "Maximal pivot tolerance for the linear solver MA57");
-#endif
-
-#ifdef HAVE_PARDISO
 
     ampl_options_list->AddAmplOption("pardiso_matching_strategy",
                                      "pardiso_matching_strategy",
@@ -1131,7 +1289,6 @@ namespace Ipopt
                                      "pardiso_out_of_core_power",
                                      AmplOptionsList::Integer_Option,
                                      "Enables out-of-core version of linear solver Pardiso");
-#endif
 
 #ifdef HAVE_WSMP
 
@@ -1171,7 +1328,7 @@ namespace Ipopt
                                              (void**)&nerror_);
 
     static const char sname_default[] = "ipopt";
-    static const char bsname_default[] = PACKAGE_STRING;
+    static const char bsname_default[] = "Ipopt " PACKAGE_VERSION;
     static const char opname_default[] = "ipopt_options";
     const char* sname;
     const char* bsname;
@@ -1253,7 +1410,7 @@ namespace Ipopt
   AmplSuffixHandler::~AmplSuffixHandler()
   {
     if (suftab_) {
-      Index n = suffix_ids_.size();
+      Index n = (Index)suffix_ids_.size();
       for (Index i=0; i<n; i++) {
         delete [] suftab_[i].name;
         suftab_[i].name = NULL;
@@ -1268,10 +1425,10 @@ namespace Ipopt
     DBG_ASSERT(asl);
     asl_ = asl;
 
-    Index n = suffix_ids_.size();
+    Index n = (Index)suffix_ids_.size();
     suftab_ = new SufDecl[n];
     for (Index i=0; i<n; i++) {
-      Index id_len = strlen(suffix_ids_[i].c_str());
+      Index id_len = (Index)strlen(suffix_ids_[i].c_str());
       suftab_[i].name = new char[id_len + 1];
       strcpy(suftab_[i].name, suffix_ids_[i].c_str());
 
@@ -1331,6 +1488,21 @@ namespace Ipopt
     return dp->u.i;
   }
 
+  std::vector<Index>
+  AmplSuffixHandler::GetIntegerSuffixValues(Index n, std::string suffix_string,
+      Suffix_Source source) const
+  {
+    std::vector<Index> ret;
+    const Index* ptr = GetIntegerSuffixValues(suffix_string, source);
+    if (ptr) {
+      ret.reserve(n);
+      for (Index i=0; i<n; i++) {
+        ret.push_back(ptr[i]);
+      }
+    }
+    return ret;
+  }
+
   const Number*
   AmplSuffixHandler::GetNumberSuffixValues(std::string suffix_string,
       Suffix_Source source) const
@@ -1358,6 +1530,22 @@ namespace Ipopt
     SufDesc* dp = suf_get(suffix_string.c_str(), kind);
     return dp->u.r;
   }
+
+  std::vector<Number>
+  AmplSuffixHandler::GetNumberSuffixValues(Index n, std::string suffix_string,
+      Suffix_Source source) const
+  {
+    std::vector<Number> ret;
+    const Number* ptr = GetNumberSuffixValues(suffix_string, source);
+    if (ptr) {
+      ret.reserve(n);
+      for (Index i=0; i<n; i++) {
+        ret.push_back(ptr[i]);
+      }
+    }
+    return ret;
+  }
+
 } // namespace Ipopt
 
 
