@@ -2,7 +2,7 @@
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpOptionsList.cpp 1245 2008-05-30 03:46:59Z andreasw $
+// $Id: IpOptionsList.cpp 1521 2009-07-15 22:26:15Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
@@ -39,6 +39,16 @@
 # endif
 #endif
 
+#ifdef HAVE_CSTRING
+# include <cstring>
+#else
+# ifdef HAVE_STRING_H
+#  include <string.h>
+# else
+#  error "don't have header file for stdlib"
+# endif
+#endif
+
 namespace Ipopt
 {
 
@@ -128,7 +138,7 @@ namespace Ipopt
                                     bool dont_print /* = false */)
   {
     char buffer[256];
-    sprintf(buffer, "%g", value);
+    Snprintf(buffer, 255, "%g", value);
 
     if (IsValid(reg_options_)) {
       SmartPtr<const RegisteredOption> option = reg_options_->GetOption(tag);
@@ -203,7 +213,7 @@ namespace Ipopt
                                     bool dont_print /* = false */)
   {
     char buffer[256];
-    sprintf(buffer, "%d", value);
+    Snprintf(buffer, 255, "%d", value);
 
     if (IsValid(reg_options_)) {
       SmartPtr<const RegisteredOption> option = reg_options_->GetOption(tag);
@@ -457,14 +467,25 @@ namespace Ipopt
 
     std::string strvalue;
     if (find_tag(tag, prefix, strvalue)) {
+      // Some people like to use 'd' instead of 'e' in floating point
+      // numbers.  Therefore, we change a 'd' to an 'e'
+      char* buffer = new char[strvalue.length()+1];
+      strcpy(buffer, strvalue.c_str());
+      for (int i=0; i<(int)strvalue.length(); ++i) {
+        if (buffer[i]=='d' || buffer[i]=='D') {
+          buffer[i] = 'e';
+        }
+      }
       char* p_end;
-      Number retval = strtod(strvalue.c_str(), &p_end);
+      Number retval = strtod(buffer, &p_end);
       if (*p_end!='\0' && !isspace(*p_end)) {
+        delete [] buffer;
         std::string msg = "Option \"" + tag +
                           "\": Double value expected, but non-numeric value \"" +
                           strvalue+"\" found.\n";
         THROW_EXCEPTION(OPTION_INVALID, msg);
       }
+      delete [] buffer;
       value = retval;
       return true;
     }
@@ -542,13 +563,13 @@ namespace Ipopt
   {
     list.erase();
     char buffer[256];
-    sprintf(buffer, "%40s   %-20s %s\n", "Name", "Value", "# times used");
+    Snprintf(buffer, 255, "%40s   %-20s %s\n", "Name", "Value", "# times used");
     list += buffer;
     for (std::map< std::string, OptionValue >::const_iterator p = options_.begin();
          p != options_.end();
          p++ ) {
-      sprintf(buffer, "%40s = %-20s %6d\n", p->first.c_str(),
-              p->second.Value().c_str(), p->second.Counter());
+      Snprintf(buffer, 255, "%40s = %-20s %6d\n", p->first.c_str(),
+               p->second.Value().c_str(), p->second.Counter());
       list += buffer;
     }
   }
@@ -557,7 +578,7 @@ namespace Ipopt
   {
     list.erase();
     char buffer[256];
-    sprintf(buffer, "%40s   %-20s %s\n", "Name", "Value", "used");
+    Snprintf(buffer, 255, "%40s   %-20s %s\n", "Name", "Value", "used");
     list += buffer;
     for (std::map< std::string, OptionValue >::const_iterator p = options_.begin();
          p != options_.end();
@@ -572,8 +593,8 @@ namespace Ipopt
         else {
           used = no;
         }
-        sprintf(buffer, "%40s = %-20s %4s\n", p->first.c_str(),
-                p->second.Value().c_str(), used);
+        Snprintf(buffer, 255, "%40s = %-20s %4s\n", p->first.c_str(),
+                 p->second.Value().c_str(), used);
         list += buffer;
       }
     }
@@ -623,14 +644,25 @@ namespace Ipopt
                            "Error setting string value read from option file.");
         }
         else if (option->Type() == OT_Number) {
+          // Some people like to use 'd' instead of 'e' in floating
+          // point numbers.  Therefore, we change a 'd' to an 'e'
+          char* buffer = new char[value.length()+1];
+          strcpy(buffer, value.c_str());
+          for (int i=0; i<(int)value.length(); ++i) {
+            if (buffer[i]=='d' || buffer[i]=='D') {
+              buffer[i] = 'e';
+            }
+          }
           char* p_end;
-          Number retval = strtod(value.c_str(), &p_end);
+          Number retval = strtod(buffer, &p_end);
           if (*p_end!='\0' && !isspace(*p_end)) {
+            delete [] buffer;
             std::string msg = "Option \"" + tag +
                               "\": Double value expected, but non-numeric option value \"" +
                               value + "\" found.\n";
             THROW_EXCEPTION(OPTION_INVALID, msg);
           }
+          delete [] buffer;
           bool result = SetNumericValue(tag, retval, false);
           ASSERT_EXCEPTION(result, OPTION_INVALID,
                            "Error setting numeric value read from file.");
